#include <zephyr/kernel.h>
#include <zephyr/device.h>
#include <zephyr/drivers/uart.h>

#include <string.h>

static const struct device *const uart_dev = DEVICE_DT_GET(DT_NODELABEL(uart1));

volatile int gNoUartRxByte = 0;

SerialDataFrame gUartTxFrame;
SerialDataFrame gUartRxFrame;
char *gUartRxFramePtr = (char *) &gUartRxFrame;


uint8_t Calculate_CRC(char *data, int count, uint16_t checksum)
{
   int iteration = 0;
   uint16_t crc  = 0;

   for (iteration = 0; iteration < count; iteration++) {
      crc += (uint16_t) data[iteration];
   }
   crc = ~crc + 1;

   if (checksum == crc) {
      return 0;
   }
   return 1;
}

/*
 * Read characters from UART until line end is detected. Afterwards push the
 * data to the message queue.
 */
void serial_cb(const struct device *dev, void *user_data)
{
   uint8_t gUartByte;

   if (!uart_irq_update(uart_dev)) {
      return;
   }

   if (!uart_irq_rx_ready(uart_dev)) {
      return;
   }

#if 1
   /* read until FIFO empty */
   while (uart_fifo_read(uart_dev, &gUartByte, 1) == 1) {
      if (gNoUartRxByte == 0 && gUartByte == RX_START_OF_FRAME) {
         gUartRxFramePtr[gNoUartRxByte++] = gUartByte;
      }
      else if (gNoUartRxByte > 0 && gNoUartRxByte < sizeof(SerialDataFrame)) {
         if (gUartByte == '\n') {
            gNoUartRxByte = 0;
            gUartDataReceived = SET;
         }
         else {
            gUartRxFramePtr[gNoUartRxByte++] = gUartByte;
         }
      }
      else {
         gNoUartRxByte = 0;
      }
#else
      if ((gUartByte == '\n' || gUartByte == '\r') && rx_buf_pos > 0) {
         /* terminate string */
         rx_buf[rx_buf_pos] = '\0';

         /* if queue is full, message is silently dropped */
         k_msgq_put(&uart_msgq, &rx_buf, K_NO_WAIT);

         /* reset the buffer (it was copied to the msgq) */
         rx_buf_pos = 0;
      } else if (rx_buf_pos < (sizeof(rx_buf) - 1)) {
         rx_buf[rx_buf_pos++] = gUartByte;
      }
      /* else: characters beyond buffer size are dropped */
#endif
   }
}


